﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MPTVAdmin2
{
    public partial class frmMain : Form
    {        
        //TODO: connAvailable really necessary
        private bool connAvailable = false;
        private List<Group> channelGroups = new List<Group>();       
        private List<Channel> mpTVAllChannels;
        private List<Channel> mpRadioAllChannels;
        private List<Channel> mpChannelsSelection;
        private MPDataWorker mpDataWorker;
        private bool activeTypeTV = true;
        private bool activeSearchName = true;        

        private void Initialize() { 
            //Load MediaPortal Channels                                    
            mpDataWorker = MPDataWorker.GetInstance();
            
            //TODO: Ask for user/pass/server (in the beginning?)
            mpDataWorker.Username = txtDBUserName.Text;
            mpDataWorker.Password = txtDBPassword.Text;
            mpDataWorker.Server =txtDBServer.Text;
            
            if (mpDataWorker.TestConnection())
            {
                connAvailable = true;
                btnAdd.Enabled = true;
                btnImportFromMediaportal.Enabled = true;
                btnExportToMediaPortal.Enabled = true;
                btnImportFromDreamBox.Enabled = true;

                //Two unique instances!
                //Load Channels
                mpTVAllChannels = mpDataWorker.GetChannels();
                mpRadioAllChannels = mpDataWorker.GetChannels(false);

                //Map channels
                mpChannelsSelection = mpDataWorker.GetChannels(activeTypeTV ? true : false);

                grdMPAllChannels.DataSource = mpChannelsSelection;
            }
            else {
                connAvailable = false;
                btnAdd.Enabled = false;
                btnImportFromMediaportal.Enabled = false;
                btnExportToMediaPortal.Enabled = false;
                btnImportFromDreamBox.Enabled = false;

                grdMPAllChannels.DataSource = null;
            }
        }

        public frmMain()
        {
            InitializeComponent();            
            Initialize();
        }

        private void txtSearch_TextChanged(object sender, EventArgs e)
        {
            //Refill the selection
            grdMPAllChannels.DataSource = null;
            mpChannelsSelection.Clear();
            
            foreach (Channel channel in (activeTypeTV ? mpTVAllChannels : mpRadioAllChannels)){
                if ((activeSearchName ? channel.Name : channel.Provider).ToUpper().Contains(txtSearch.Text.ToUpper())){
                    mpChannelsSelection.Add(channel);
                }
            }
           
            grdMPAllChannels.DataSource = mpChannelsSelection;
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            
            if (lstGroups.SelectedIndex != -1)
            {             
                for (int i = 0; i < grdMPAllChannels.SelectedRows.Count; i++)
                {
                    //Walk through the selected rows                    
                    Channel channel = (Channel)grdMPAllChannels.SelectedRows[i].DataBoundItem;
                    //Check if channel isn't already a member of the group
                    if (!channelGroups[lstGroups.SelectedIndex].Channels.Contains(channel))
                    {
                        channelGroups[lstGroups.SelectedIndex].Channels.Add(channel);
                    }
                }
                RefreshLists();
            }
        }

        private void tabControl_SelectedIndexChanged(object sender, EventArgs e)
        {
            Initialize();
        }

        private void radProvider_CheckedChanged(object sender, EventArgs e)
        {
            activeSearchName = false;
        }

        private void radName_CheckedChanged(object sender, EventArgs e)
        {
            activeSearchName = true;
        }

        private void btnImportFromDreamBox_Click(object sender, EventArgs e)
        {
            String filter = (activeTypeTV ? "bouquets.tv |bouquets.tv" : "bouquets.radio |bouquets.radio");
            dlgOpenFile.FileName = "";
            dlgOpenFile.Filter = filter;
            if (dlgOpenFile.ShowDialog() == DialogResult.OK) {
                channelGroups = Importer.readBouqetsTV(dlgOpenFile.FileName);
            }

            //activeTypeTV decides to map on Radio/TV
            channelGroups = Importer.mapChannels(channelGroups, activeTypeTV ? mpTVAllChannels : mpRadioAllChannels, 
                                chkDeleteUnknown.Checked, chkDeleteEmptyNames.Checked, chkDeleteEmptyGroups.Checked);
            RefreshLists();
            
        }

        private void lstGroups_SelectedIndexChanged(object sender, EventArgs e)
        {
            //Check if the list isn't empty          
            if (channelGroups.Count != 0 && lstGroups.SelectedItem != null)
            {
                lstChannels.DataSource = ((Group)lstGroups.SelectedItem).Channels;
                txtGroupName.Text = ((Group)lstGroups.SelectedItem).Name;
            }
        }

        private void btnClearAllTV_Click(object sender, EventArgs e)
        {
            ClearLists();
        }

        private void ClearLists(){            
            channelGroups.Clear();
            lstGroups.DataSource = null;
            lstChannels.DataSource = null;
            txtGroupName.Text = "";
        }

        private void cntMenuGroupDelete_Click(object sender, EventArgs e)
        {
            //Delete selected item(s).
            channelGroups.Remove((Group) lstGroups.SelectedItem);
            if (channelGroups.Count == 0){
                ClearLists();
            }
            RefreshLists();               
        }

        private void txtGroupName_TextChanged(object sender, EventArgs e)
        {
            if (lstGroups.SelectedIndex != -1)
            {
                channelGroups[lstGroups.SelectedIndex].Name = txtGroupName.Text;
                RefreshLists();
            }
        }


        private void RefreshLists(){
            lstChannels.DataSource = null;
            lstChannels.DisplayMember = "Name";
            lstGroups.DataSource = null;
            lstGroups.DataSource = channelGroups;
            lstGroups.DisplayMember = "NameAndCount";
        }

        private void cntMenuGroupsAdd_Click(object sender, EventArgs e)
        {
            Group newGrp = new Group("New Group");
            channelGroups.Add(newGrp);
            RefreshLists();
        }

        private void cntMenuGroupsMoveUp_Click(object sender, EventArgs e)
        {
            //Move UP            
            int index = lstGroups.SelectedIndex;
            if (index != 0 && index != -1)
            {                
                Group grp = (Group)channelGroups[index];
                channelGroups.RemoveAt(index);
                channelGroups.Insert(index - 1, grp);
                RefreshLists();
                lstGroups.SelectedIndex = index - 1;
            }
        }

        private void cntMenuGroupsMoveDown_Click(object sender, EventArgs e)
        {
            //Move Down
            int index = lstGroups.SelectedIndex;
            if (index != channelGroups.Count - 1 && index != -1)
            {
                Group grp = (Group)channelGroups[index];
                channelGroups.RemoveAt(index);
                channelGroups.Insert(index + 1, grp);
                RefreshLists();
                lstGroups.SelectedIndex = index + 1;
            }
        }

        private void cntMenuChannelsDelete_Click(object sender, EventArgs e)
        {
            //Delete selected item(s).
            channelGroups[lstGroups.SelectedIndex].Channels.Remove((Channel) lstChannels.SelectedItem);
            RefreshLists();
        }

        private void btnImportFromMediaportal_Click(object sender, EventArgs e)
        {
            channelGroups = mpDataWorker.GetGroups(activeTypeTV);
            RefreshLists();
        }

        private void cntMenuChannelsMoveDown_Click(object sender, EventArgs e)
        {
            //Move Down           
            int index = lstChannels.SelectedIndex;
            if (index != channelGroups[lstGroups.SelectedIndex].Channels.Count - 1 && index != -1)
            {
                Channel chnl = (Channel)channelGroups[lstGroups.SelectedIndex].Channels[index];
                channelGroups[lstGroups.SelectedIndex].Channels.RemoveAt(index);
                channelGroups[lstGroups.SelectedIndex].Channels.Insert(index + 1, chnl);
                RefreshLists();
                lstChannels.SelectedIndex = index + 1;
            }

        }

        private void cntMenuChannelsMoveUp_Click(object sender, EventArgs e)
        {
            //Move UP            
            int index = lstChannels.SelectedIndex;
            if (index != 0 && index != -1)
            {
                Channel chnl = (Channel)channelGroups[lstGroups.SelectedIndex].Channels[index];
                channelGroups[lstGroups.SelectedIndex].Channels.RemoveAt(index);
                channelGroups[lstGroups.SelectedIndex].Channels.Insert(index - 1, chnl);
                RefreshLists();
                lstChannels.SelectedIndex = index - 1;
            }
        }

        private void btnExportToMediaPortal_Click(object sender, EventArgs e)
        {
            mpDataWorker.ExportChannelsToMP(channelGroups, chkDeleteExistingGroups.Checked, activeTypeTV);
            MessageBox.Show("Done!");
        }

        private void btnTestConnection_Click(object sender, EventArgs e)
        {
            mpDataWorker.Username = txtDBUserName.Text;
            mpDataWorker.Password = txtDBPassword.Text;
            mpDataWorker.Server = txtDBServer.Text;
            if (mpDataWorker.TestConnection())
            {
                //Jippie
                MessageBox.Show("Connection succesfull");
                connAvailable = true;
                Initialize();
            }
            else { 
                //Error
                MessageBox.Show("Connection failed");
                connAvailable = false;
                Initialize();
            }
        }

        private void txtDBUserName_TextChanged(object sender, EventArgs e)
        {
            mpDataWorker.Username = txtDBUserName.Text;
        }

        private void txtDBPassword_TextChanged(object sender, EventArgs e)
        {
            mpDataWorker.Password = txtDBPassword.Text;
        }

        private void txtDBServer_TextChanged(object sender, EventArgs e)
        {
            mpDataWorker.Server = txtDBServer.Text;
        }

        private void cmbWorking_SelectedIndexChanged(object sender, EventArgs e)
        {
            //TV or Radio
            if (cmbWorking.SelectedItem.ToString() == "Radio")
            {
                activeTypeTV = false;
                if (mpDataWorker.TestConnection())
                {
                    mpChannelsSelection = mpDataWorker.GetChannels(false);
                    grdMPAllChannels.DataSource = mpChannelsSelection;
                }
            }
            else
            {
                activeTypeTV = true;                
                if (mpDataWorker.TestConnection())
                {
                    mpChannelsSelection = mpDataWorker.GetChannels();
                    grdMPAllChannels.DataSource = mpChannelsSelection;
                }
            }
            ClearLists();
        }

        private void lstChannels_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                cntMenuChannelsDelete_Click(sender, e);
            }
            if (e.KeyCode == Keys.U){
                cntMenuChannelsMoveUp_Click(sender, e);                
            }
            if (e.KeyCode == Keys.D){
                cntMenuChannelsMoveDown_Click(sender, e);
            }
        }

        private void lstGroups_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
               cntMenuGroupDelete_Click(sender, e);
            }
            if (e.KeyCode == Keys.U){
                cntMenuGroupsMoveUp_Click(sender, e);
            }
            if (e.KeyCode == Keys.D){
                cntMenuGroupsMoveDown_Click(sender, e);
            }        
        }

    }
}
